'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _tools = require('views/utils/tools');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// similar to CountdownTimer in 'views/components/main/parts/countdown-timer.es', but it counts up

class CountupTimer extends _react.Component {
  constructor(props) {
    super(props);
    this.state = {
      startTime: this.props.startTime
    };

    this.componentDidMount = () => {
      this.startTick();
    };

    this.componentWillReceiveProps = nextProps => {
      if (nextProps.countdownId !== this.props.countdownId) {
        this.stopTick();
      }
      if (nextProps.startTime !== this.state.startTime) {
        this.setState({ startTime: nextProps.startTime });
        this.timeElapsed = this.constructor.getTimeElapsed(nextProps.startTime);
      }
    };

    this.shouldComponentUpdate = (nextProps, nextState) => nextProps.countdownId !== this.props.countdownId || nextState.startTime !== this.state.startTime;

    this.componentDidUpdate = () => {
      this.startTick // Doesn't matter if it didn't stop
      ();
    };

    this.componentWillUnmount = () => {
      this.stopTick();
    };

    this.startTick = () => {
      window.ticker.reg(this.props.countdownId, this.tick);
    };

    this.stopTick = () => {
      window.ticker.unreg(this.props.countdownId);
    };

    this.tick = currentTime => {
      // const actualElapsed = this.constructor.getTimeElapsed(this.state.startTime, currentTime)
      // if (Math.abs(this.timeElapsed - actualElapsed) > 2) {
      //   this.timeElapsed = actualElapsed
      // }
      this.timeElapsed = this.constructor.getTimeElapsed(this.state.startTime, currentTime);
      if (this.timeElapsed < 0) {
        this.stopTick();
      }
      if (this.state.startTime >= 0) {
        try {
          if (this.textLabel) {
            this.textLabel.textContent = (0, _tools.resolveTime)(this.timeElapsed) || (0, _tools.resolveTime)(0);
          }
          if (this.props.tickCallback) {
            this.props.tickCallback(this.timeElapsed);
          }
          if (this.timeElapsed < 1 && this.props.startCallback) {
            this.props.startCallback();
          }
        } catch (error) {
          console.error(error.stack);
        }
      }
      this.timeElapsed += 1;
    };

    this.timeElapsed = this.constructor.getTimeElapsed(this.props.startTime);
  }

  render() {
    return _react2.default.createElement(
      'span',
      { ref: ref => {
          this.textLabel = ref;
        } },
      (0, _tools.resolveTime)(this.timeElapsed)
    );
  }
}
exports.default = CountupTimer;
CountupTimer.propTypes = {
  countdownId: _react.PropTypes.string.isRequired, // UNIQUE ID to register to window.ticker
  startTime: _react.PropTypes.number, // startTime in ms
  tickCallback: _react.PropTypes.func, // callback function for each second
  startCallback: _react.PropTypes.func // callback function when starting to count up
};

CountupTimer.getTimeElapsed = (startTime, currentTime = Date.now()) => {
  if (startTime <= 0) {
    return -1;
  } else if (startTime > currentTime) {
    return 0;
  }
  return Math.round((currentTime - startTime) / 1000);
};

CountupTimer.defaultProps = {
  startTime: -1,
  tickCallback: null,
  startCallback: null
};
module.exports = exports['default'];